#include <catch2/catch_approx.hpp>
#include <catch2/catch_test_macros.hpp>

#include <Kokkos_Core.hpp>

#include <utils/PugsAssert.hpp>
#include <utils/Types.hpp>

#include <algebra/TinyMatrix.hpp>
#include <analysis/Polynomial.hpp>
#include <analysis/PolynomialBasis.hpp>
#include <mesh/CartesianMeshBuilder.hpp>
#include <mesh/DiamondDualConnectivityManager.hpp>
#include <mesh/DiamondDualMeshManager.hpp>
#include <scheme/DiscontinuousGalerkin1DTools.hpp>
#include <scheme/ODEGD1D.hpp>

// Instantiate to ensure full coverage is performed
template class Polynomial<0>;
template class Polynomial<1>;
template class Polynomial<2>;
template class Polynomial<3>;
template class Polynomial<4>;
template class Polynomial<5>;

// clazy:excludeall=non-pod-global-static

TEST_CASE("Discontinuous-Galerkin-1D", "[discontinuous-galerkin-1d]")
{
  SECTION("Elementary-tests")
  {
    TinyVector<1> a{0};
    TinyVector<1> b{1};
    TinyVector<1, uint64_t> nbcells{10};
    // CartesianMeshBuilder mesh_builder = CartesianMeshBuilder(a, b, nbcells);
    // auto mesh                         = mesh_builder.mesh();
    // DiscontinuousGalerkin1D<2>{mesh};
    REQUIRE_NOTHROW(DiscontinuousGalerkin1DTools<2>());
    DiscontinuousGalerkin1DTools dg = DiscontinuousGalerkin1DTools<1>();
    Polynomial<1> U;
    TinyVector<2> F({3, 2});
    PolynomialBasis<1> Basis;
    Basis.build(BasisType::canonical);
    dg.elementarySolve(U, F, Basis, -1, 1);
    REQUIRE(U == Polynomial<1>{{3. / 2, 3}});
    Polynomial<2> U2;
    TinyVector<3> F2({1, 1, 1});
    PolynomialBasis<2> Basis2;
    Basis2.build(BasisType::canonical);
    DiscontinuousGalerkin1DTools dg2 = DiscontinuousGalerkin1DTools<2>();
    dg2.elementarySolve(U2, F2, Basis2, -1, 1);
    REQUIRE(integrate(U2, -1, 1) == Catch::Approx(1).epsilon(1E-14));
    REQUIRE(integrate(U2 * Polynomial<2>{{0, 0, 1}}, -1, 1) == 1.);
    dg2.elementarySolve(U2, F2, Basis2, 0, 1);
    REQUIRE(integrate(U2 * Polynomial<2>{{0, 0, 1}}, 0, 1) == Catch::Approx(1).epsilon(1E-14));
  }
}
